@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009,2011,2012,2013,2014,2015,2017 Red Hat, Inc.
- * 
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -42,7 +42,7 @@
 #include <krb5.h>
 
 #ifdef CM_USE_IDN
-#include <idna.h>
+#include <idn2.h>
 #endif
 
 #include "certext.h"
@@ -203,45 +203,6 @@ cm_ms_template_template[] = {
 	{0, 0, NULL, 0},
 };
 
-/* RFC 5280, 4.1 */
-const SEC_ASN1Template
-cm_certext_cert_extension_template[] = {
-	{
-	.kind = SEC_ASN1_SEQUENCE,
-	.offset = 0,
-	.sub = NULL,
-	.size = sizeof(CERTCertExtension),
-	},
-	{
-	.kind = SEC_ASN1_OBJECT_ID,
-	.offset = offsetof(CERTCertExtension, id),
-	.sub = NULL,
-	.size = sizeof(SECItem),
-	},
-	{
-	.kind = SEC_ASN1_BOOLEAN,
-	.offset = offsetof(CERTCertExtension, critical),
-	.sub = NULL,
-	.size = sizeof(SECItem),
-	},
-	{
-	.kind = SEC_ASN1_OCTET_STRING,
-	.offset = offsetof(CERTCertExtension, value),
-	.sub = NULL,
-	.size = sizeof(SECItem),
-	},
-	{0, 0, NULL, 0},
-};
-const SEC_ASN1Template
-cm_certext_sequence_of_cert_extension_template[] = {
-	{
-	.kind = SEC_ASN1_SEQUENCE_OF,
-	.offset = 0,
-	.sub = cm_certext_cert_extension_template,
-	.size = sizeof(CERTCertExtension **),
-	},
-};
-
 /* Windows 2000-style UPN */
 static unsigned char oid_ms_upn_name_bytes[] = {0x2b, 0x06, 0x01, 0x04, 0x01, 0x82, 0x37, 0x14, 0x02, 0x03};
 static const SECOidData oid_ms_upn_name = {
@@ -1620,9 +1581,9 @@ cm_certext_build_certificate_template(
 		return NULL;
 
 	int i = 0;
-	char *saveptr, *endptr;
+	char *saveptr, *endptr, *part;
 	for (
-		char *part = strtok_r(template_spec_dup, ":", &saveptr);
+		part = strtok_r(template_spec_dup, ":", &saveptr);
 		part != NULL;
 		part = strtok_r(NULL, ":", &saveptr)
 	) {
@@ -1745,9 +1706,12 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 	CERTCertExtension ext[13], *exts[14], **exts_ptr;
 	SECOidData *oid;
 	SECItem *item, encoded;
+	/* X509v3 extension's critical element has an implicit default,
+	 * see https://pagure.io/certmonger/issue/223
+	 */
 	SECItem der_false = {
-		.len = 1,
-		.data = (unsigned char *) "\000",
+		.len = 0,
+		.data = NULL,
 	};
 	SECItem der_true = {
 		.len = 1,
@@ -1960,7 +1924,7 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 	/* Encode the sequence. */
 	memset(&encoded, 0, sizeof(encoded));
 	if (i > 1) {
-		template = cm_certext_sequence_of_cert_extension_template;
+		template = CERT_SequenceOfCertExtensionTemplate;
 		if (SEC_ASN1EncodeItem(arena, &encoded, &exts_ptr,
 				       template) == &encoded) {
 			*extensions = talloc_memdup(entry, encoded.data,
