@@ -363,7 +363,6 @@ cm_store_time_from_timestamp(const char *timestamp)
 	buf[2] = '\0';
 	stamp.tm_min = atoi(buf);
 	memcpy(buf, timestamp + i, 2);
-	i += 2;
 	buf[2] = '\0';
 	stamp.tm_sec = atoi(buf);
 	t = timegm(&stamp);
@@ -530,8 +529,6 @@ cm_store_hex_to_bin(const char *serial, unsigned char *buf, int length)
 	const char *p, *q, *chars = "0123456789abcdef";
 	unsigned char *b, u;
 
-	p = serial;
-	b = buf;
 	u = 0;
 	for (p = serial, b = buf;
 	     ((*p != '\0') && ((b - buf) < length));
@@ -606,7 +603,6 @@ cm_store_canonicalize_path(void *parent, const char *path)
 		for (p = tmp; *p != '\0'; p++) {
 			if ((strncmp(p, "/.", 2) == 0) &&
 			    ((p[2] == '/') || (p[2] == '\0'))) {
-				q = p - 1;
 				memmove(p, p + 2, strlen(p + 2) + 1);
 			}
 		}
