@@ -12,7 +12,7 @@ run_certutil -d "$tmpdir" -S -n keyi$size \
 	-s "cn=T$size" -c "cn=T$size" \
 	-x -t u -k ec -q $size
 # Export the key.
-pk12util -d "$tmpdir" -o $size.p12 -W "" -n "keyi$size" > /dev/null 2>&1
+pk12util -C AES-128-CBC -c AES-128-CBC -d "$tmpdir" -o $size.p12 -W "" -n "keyi$size" > /dev/null 2>&1
 openssl pkcs12 -in $size.p12 -out key.$size -passin pass: -nodes -nocerts > /dev/null 2>&1 | ( grep -v '^MAC verified OK$' || : )
 # Read the public key and cache it.
 cat > entry.openssl.$size <<- EOF
@@ -42,8 +42,8 @@ grep ^minicert= entry.nss.$size | sed s,^minicert=,, | base64 -d > minicert.nss.
 openssl x509 -out minicert.nss.$size.pem -in minicert.nss.$size -inform der
 # The RSA tests already verify the contents of the requests, so we really only
 # need to care about the signatures passing verification.
-openssl req   -verify -noout < csr.nss.$size 2>&1
-openssl req   -verify -noout < csr.openssl.$size 2>&1
+openssl req   -verify -noout < csr.nss.$size 2>&1 | sed 's/Certificate request self-signature //'
+openssl req   -verify -noout < csr.openssl.$size 2>&1 | sed 's/Certificate request self-signature //'
 openssl spkac -verify -noout < spkac.nss.$size 2>&1
 openssl spkac -verify -noout < spkac.openssl.$size 2>&1
 openssl verify -CAfile minicert.openssl.$size.pem minicert.openssl.$size.pem 2>&1
